#define DB_INFO 0
#define DB_ENABLED 0


// getmsec.cc
// get the number of milliseconds

// uint32_t getmsec()
//				gets millisecond value (after undefined time point)
// uint32_t diffmsec( startmsec )
//				gets elapsed time since that point
// uint32_t diffmsec( uint32_t startmsec, uint32_t actmsec )
//				calculates time msec from startmsec to actmsec
// uint32_t addmsec( uint32_t startmsec, uint32_t elpmsec )
//				calculates the getmsec() return value
//				elpmsec after startmsec

#include <heport.h>
#include <helq.h>
#include <hedeb.h>
#if ! defined( __TURBOCDOS__ ) && defined( __TURBOC__ )
#include <bios.h>	// biostime
#endif
#if defined( MSVC10 ) || defined( MDOS10 )
#include <dos.h>	// _disable _enable
#endif

#if	 defined ( MDOS10 )
#include <bios.h>	// _bios_timeofday
#endif



#if defined( MSVC40 ) || defined( MDOS40 )
#include <mapidefs.h>	// FILETIME ...
#endif

#ifdef OSK

FUNCT	uint32_t	getmsec() /* get count of milliseconds after midnight  OSK */
{
int atime ;
int adate ;
short aday ;
int atick ;	 /* Highbyte : tick per second , Lowbyte : current clock tick */
uint32_t restmsec ;	   /* rev:abc */

_sysdate( 3, &atime, &adate, &aday, &atick );

restmsec = 1000 * ( atick & 0x7FFF ) / ( ( atick >> 16 ) & 0x7FFF );
return 1000L * atime + restmsec ;		/* Milliseconds after midnight */
}


/*+****************************************************************************
*
*		Procedure:		diffmsec
*
*		Parameters:		uint32_t : start time (milliseconds after midnight)
*
*		Return Value:	uint32_t : 0 .. (24*60*60*1000-1) count of milliseconds
*						after start, correct even of midnight is passed
*
*		Description:	get count of milliseconds after a spezified time
*
*-****************************************************************************/

uint32_t diffmsec( uint32_t startmsec )					/* OSK */
{
long	retval ;
retval = (long)(getmsec() - startmsec) ;
if ( retval < 0 )
	{	/* special case : start was before midnight */
	retval += 24L*60L*60L*1000L ;
	}
return retval ;
}


/*+****************************************************************************
*
*		Procedure:		diffmsec
*
*		Parameters:		uint32_t : start time (milliseconds after midnight)
*
*		Return Value:	uint32_t : 0 .. (24*60*60*1000-1) count of milliseconds
*						after start, correct even of midnight is passed
*
*		Description:	get count of milliseconds after a spezified time
*
*-****************************************************************************/

uint32_t diffmsec( uint32_t startmsec, uint32_t actmsec )					/* OSK */
{
long	retval ;
retval = (long)(actmsec - startmsec) ;
if ( retval < 0 )
	{	/* special case : start was before midnight */
	retval += 24L*60L*60L*1000L ;
	}
return retval ;
}


FUNCT uint32_t addmsec( uint32_t startmsec, uint32_t offset )				/* OSK */
{
return	startmsec + offset ;
}


#endif // OSK





#if defined( DOSGNU )
#include <dos.h>
#endif

#include <heport.h>
#include <hegettim.h>

#if defined( DOSGNU )

FUNCT	uint32_t	getmsec()		// DOSGNU
// get count of milliseconds after midnight
{
struct time now ;

gettime(&now) ;
return (((uint32_t)now.ti_hour * 60 + now.ti_min) * 60 + now.ti_sec ) * 1000 ;
}

FUNCT uint32_t diffmsec( uint32_t startmsec )		// DOSGNU
{
long	retval ;
retval = (long)(getmsec() - startmsec) ;
if ( retval < 0 )
	{	/* special case : start was before midnight */
	retval += 24L*60L*60L*1000L ;
	}
return retval ;
}

FUNCT uint32_t diffmsec( uint32_t startmsec, uint32_t actmsec )		// DOSGNU
{
long	retval ;
retval = (long)(actmsec - startmsec) ;
if ( retval < 0 )
	{	/* special case : start was before midnight */
	retval += 24L*60L*60L*1000L ;
	}
return retval ;
}

FUNCT uint32_t addmsec( uint32_t startmsec, uint32_t offset )	// DOSGNU
{
return	startmsec + offset ;
}

#endif									// DOSGNU



// this function is verified by qq/qqnsec.cpp
#if defined( __TURBOC__ ) || defined( MSVC10 ) || defined( MDOS10 )
// Turbo C for DOS
// Turbo C for Windows
// MSVC 1.0 for Windows 3.1

FUNCT	uint32_t	getmsec() /* get count of milliseconds after midnight */
{
uint32_t far *p ;
long long_x ;
uint32_t ticks ;
uint32_t ticksearly ;
uint8_t ctrlow , ctrhigh ;	// content of 16 bit counter
uint16_t ctr ;
static uint32_t lastmsec ;		// correction
uint32_t ldifmsec ;
#if defined( __TURBOC__ ) || defined( MDOS10 )
double dmsec ;
#endif

double dticks ;
double tickrest ;
uint32_t msec ;


p = (uint32_t far *)0x0000046C ;

disable();			// Interrupt off

#if defined( __TURBOCDOS__ )
ticksearly = *p ;		// DOS Only
#elif defined( __TURBOC__ )
ticksearly = biostime( 0, 0L );
#elif  defined ( MDOS10 )
_bios_timeofday( _TIME_GETCLOCK, &long_x );
ticksearly = long_x ;
#elif  defined ( MSVC10 )
ticksearly = GetTickCount() ;	// Windows
#endif

outportb( 0x43, 0 );	// Latch counter
ctrlow	= inportb( 0x40 );
ctrhigh = inportb( 0x40 );
enable();			// interrupt on

dticks	 = ticksearly ;


disable();


#if defined( __TURBOCDOS__ )
ticks = *p ;	   // DOS Only
#elif defined( __TURBOC__ )
ticks = biostime( 0, 0L );
#elif  defined ( MDOS10 )
_bios_timeofday( _TIME_GETCLOCK, &long_x );
ticks = long_x ;
#elif  defined ( MSVC10 )
ticks = GetTickCount() ;	// Windows
#endif

enable();


if ( ticks	!= ticksearly )
	{
#if defined( __TURBOC__ ) || defined( MDOS10 )
	dticks	 = ticks ;
	dmsec	 = dticks * 54.95681341719 ;
	msec	 = (uint32_t)dmsec ;
#else
	msec = ticks ;
#endif
	DBprintf(( "ticks changed early=%lu ticks=%lu", ticksearly, ticks ));
	}
else
	{	// ( ticks == ticksearly )
		// dticks = ticks ;
	ctr = ~ (( ctrhigh << 8 ) + ctrlow) ;
	tickrest = (double)( ctr );
#if defined( __TURBOC__ ) || defined( MDOS10 )
	dticks	+= tickrest / 65536.0 ;
	dmsec	 = dticks * 54.95681341719 ;
	msec	 = (uint32_t)dmsec ;
#else
	msec = ticks + (uint32_t)( tickrest / 54.95681341719 ) ;
#endif
	DBprintf(( "ticks=%lu tickrest=%12.6f", ticks, tickrest ));
	}
msec %= 24L*60L*60L*1000L ;
if ( msec > lastmsec )
	{
	ldifmsec = msec - lastmsec ;
	}
else
	{
	ldifmsec = msec - lastmsec + (24L*60L*60L*1000L) ;
	}

if ( ldifmsec > (24L*60L*60L*1000L - 1000L ) )
	{
	return lastmsec ;
	}
else
	{
	lastmsec = msec ;
	return msec ;
	}
}



FUNCT uint32_t diffmsec( uint32_t startmsec )		// __TURBOC__ and MSVC10
{
long	retval ;
retval = (long)(getmsec() - startmsec) ;
if ( retval < 0 )
	{	/* special case : start was before midnight */
	retval += 24L*60L*60L*1000L ;		// maximum is 86400000L
	}
return retval ;
}

uint32_t diffmsec( uint32_t startmsec, uint32_t actmsec )
{
long	retval ;
retval = (long)(actmsec - startmsec) ;
if ( retval < 0 )
	{	/* special case : start was before midnight */
	retval += 24L*60L*60L*1000L ;		// maximum is 86400000L
	}
return retval ;
}

FUNCT uint32_t addmsec( uint32_t startmsec, uint32_t difference )
{
uint32_t	 retval ;
retval = startmsec + difference ;
if ( retval > 24L*60L*60L*1000L )
	{			// special case : Overflow past midnight
	retval -= 24L*60L*60L*1000L ;		// maximum is 86400000L
	}
return retval ;
}

#endif						// __TURBOC__ and MSVC10




// /////////////////////////////////////////////////////////////

#if defined( MDOS40 ) || defined( MSVC40 ) || defined( MDOS50 ) // MSVC40

FUNCT	uint32_t	getmsec() /* get count of milliseconds after midnight */
{
static uint32_t start_milliseconds = 0x7FFFFFFF ;
register uint32_t actual_milliseconds ;	// after start of Win95

actual_milliseconds = GetTickCount() ;
if ( start_milliseconds == 0x7FFFFFFF )
	{
	struct _SYSTEMTIME now ;

	GetLocalTime(&now) ;
	start_milliseconds =
		((( (uint32_t)now.wHour * 60)
		  + now.wMinute) * 60L
		 + now.wSecond) * 1000L
		+ now.wMilliseconds - actual_milliseconds ;
	}
return ( start_milliseconds + actual_milliseconds ) % 86400000L ;
}




/*+****************************************************************************
*
*		Procedure:		diffmsec
*
*		Parameters:		uint32_t : start time (milliseconds after midnight)
*
*		Return Value:	uint32_t : 0 .. (24*60*60*1000-1) count of milliseconds
*						after start, correct even of midnight is passed
*
*		Description:	get count of milliseconds after a spezified time
*
*-****************************************************************************/

uint32_t diffmsec( uint32_t startmsec )
{
return ( getmsec() - startmsec ) % 86400000L  ;
}

uint32_t diffmsec( uint32_t startmsec, uint32_t actmsec )
{
return ( actmsec - startmsec ) % 86400000L	;
}

uint32_t addmsec( uint32_t startmsec, uint32_t actmsec )
{
return ( actmsec + startmsec ) % 86400000L	;
}


#endif // defined( _WIN31_ ) || defined( MDOS40 )


#ifdef LINUX
#include <sys/time.h>
// #include <linux/time.h>
static struct timezone tz = { 0 ,	// minutes west of greenwich
			0 } ;	// Type of daylight saving
FUNCT uint32_t getmsec( void )				// LINUX
{
struct timeval lintv ;

gettimeofday( &lintv, &tz); // Mit timezone
return lintv.tv_sec * 1000 + lintv.tv_usec / 1000 ;
}


FUNCT uint32_t diffmsec(uint32_t start)				// LINUX
{

return getmsec() - start ;
}


FUNCT uint32_t diffmsec(uint32_t start, uint32_t actmsec )		// LINUX
{

return actmsec - start ;
}


FUNCT uint32_t addmsec( uint32_t actmsec, uint32_t offset )		// LINUX
{

return actmsec + offset ;
}

#endif




// /////////////////////////////////////////////////////////////
#ifdef AVR
FUNCT uint32_t getmsec()
{
extern uint32_t g_msec ;

return g_msec ;
}

#endif
